"""cli
Command Line Interface for euler-workbench.

Copyright (C) 2014  Stijn Wouters  <w.stijn@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Last modified: 10 August 2014.
By: Stijn Wouters.
"""
import os
import pkgutil
import importlib

import click

from eulerworkbench import __version__


class Cli(click.MultiCommand):
    """Command Line Interface for your Project Euler.

    In order to get a plugin registered, you must install the plugins into
    PLUGIN_PACKAGE. The plugin must be a single module, with the (root) command
    as a function called `cli`.
    """

    # plugin folder, where all plugins are stored
    PLUGIN_FOLDER = os.path.join(os.path.dirname(__file__), 'plugins')

    # full name of the plugin package
    PLUGIN_PACKAGE = 'eulerworkbench.plugins'

    def list_commands(self, context):
        """List all available commands.

        Returns a list of commands in sorted order.
        """
        modules = pkgutil.iter_modules(path=[Cli.PLUGIN_FOLDER])

        commands = (name for _, name, _ in modules)

        return sorted(commands)

    def get_command(self, context, name):
        """Get the command from the plugins folder to execute.
        """
        try:
            module = importlib.import_module(Cli.PLUGIN_PACKAGE + '.' + name)
            return module.main
        except ImportError:
            # couldn't find the given command
            msg = 'Couldn\'t find plugin "{}"'.format(name)
            raise click.UsageError(msg, ctx=context)
        except AttributeError:
            # happens only when listing commands
            return None


@click.group(cls=Cli)
@click.version_option(prog_name='euler-workbench',
                      version='.'.join(str(i) for i in __version__))
def main():
    """Command Line application for your Project Euler.
    """
    pass
