"""eulerworkbench.plugins.plugin
Plugin manager package, for managing eulerworkbench plugins.

Copyright (C) 2014  Stijn Wouters  <w.stijn@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Last modified: 10 August 2014.
By: Stijn Wouters.
"""
import os
import shutil

import click
from eulerworkbench import cli


__version__ = (0, 1, 0)


@click.group('plugin')
@click.version_option(prog_name='euler-workbench plugin',
                      version='.'.join(str(i) for i in __version__))
def main():
    """Manage plugins.
    """
    pass


@main.command()
@click.argument('path', required=True,
                type=click.Path(exists=True, resolve_path=True))
@click.option('--editable', '-e', is_flag=True, help='Install in develop mode')
def install(path, editable):
    """Install plugin.
    """
    click.echo('Checking on existing plugins...')

    plugin_path = os.path.join(cli.Cli.PLUGIN_FOLDER, os.path.basename(path))

    if os.path.exists(plugin_path):
        msg = 'Plugin is already installed.'
        raise click.BadParameter(msg)
    else:
        click.echo('Installing plugin...')

        if editable:
            os.symlink(path, plugin_path)
        else:
            shutil.copy(path, plugin_path)

        click.echo('Done')


@main.command()
@click.argument('plugin', required=True)
def uninstall(plugin):
    """Uninstall plugin.
    """
    click.echo('Searching plugins...')

    plugin_path = os.path.join(cli.Cli.PLUGIN_FOLDER, plugin + '.py')

    plugin_exist = os.path.exists(plugin_path)

    if plugin_exist:
        click.echo('Remove plugin...')
        os.remove(plugin_path)
        click.echo('Done')
    else:
        msg = 'Plugin is not installed.'
        raise click.BadParameter(msg)
